/* translation of file "petrinet-patterns-shortcuts.k" */
/* generated by:
 *  @(#)$Author: Kimwitu++ version 2.3.10 (C) 1998-2007 Humboldt-University of Berlin $
 */
#define KC_FUNCTIONS_petrinet_patterns_shortcuts

#include <stdlib.h>
#include "ast-system-k.h"
#include "petrinet-patterns-shortcuts.h"
namespace kc { }
using namespace kc;
/* included stuff */
#line 22 "petrinet-patterns-shortcuts.k"
/*!
 * \file    petrinet-patterns-shortcuts.cc
 *
 * \brief   Petri net patterns for shortcuts
 *
 * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
 *          last changes of: \$Author: znamirow $
 * 
 * \since   2007-03-06
 *
 * \date    \$Date: 2007/04/11 12:58:43 $
 * 
 * \note    This file is part of the tool BPEL2oWFN and was created during the
 *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
 *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
 *
 * \note    This file was created using Kimwitu++ reading file
 *          petrinet-patterns-shortcuts.k.
 *          See http://site.informatik.hu-berlin.de/kimwitu++ for details.
 *
 * \version \$Revision: 1.21 $
*/

#line  38 "petrinet-patterns-shortcuts.cc"
#line 62 "petrinet-patterns-shortcuts.k"
#include <iostream>
#include <cmath>

#include "ast-details.h"
#include "globals.h"
#include "petrinet.h"

extern PetriNet PN;

using std::cerr;
using std::endl;

#line  52 "petrinet-patterns-shortcuts.cc"
#line 80 "petrinet-patterns-shortcuts.k"
/*!
 * \fn      Transition *kc::throwFault(Place *p1, Place *p2, string p1name, string prefix, integer id, int negativeControlFlow, bool preventFurtherFaults = true)
 *
 * \brief   generates transition and places to throw a fault
 *
 *          This functions generates a subnet to model the throwing of a fault.
 *          It models faults in any control flow of the BPEL process as well as
 *          in any patterns used.
 *
 * \param   p1  the place in positive control flow from which the control flow
 *              enters the negative control flow
 *
 * \param   p2  the place of the pattern on which a token shall be produced which
 *              can only be removed by stopping the pattern
 *
 * \param   p1name  name of place p1 which is used to label the generated
 *                  transitions (e.g. if p1 is called "running" generated
 *                  transitions end with this name (e.g. "throwFault.running")
 *
 * \param   prefix  prefix of the pattern to label generated places and
 *                  transitions
 *
 * \param   id  identifier of the caller activity
 *
 * \param   negativeControlFlow  signals where the activity is located:
 *                                - 0: inside a scope or the process
 *                                - 1: inside a fault handler
 *                                - 2: inside a compensation handler
 *
 * \param   preventFurtherFaults  controls what happens to further faults
 *                                 - true: these faults are prevented (standard)
 *                                 - false: these faults are suppressed
 *
 * \return  a pointer to the (first) generated fault transition
 *
 * \ingroup patterns
*/

#line  92 "petrinet-patterns-shortcuts.cc"
#line 216 "petrinet-patterns-shortcuts.k"
/*!
 * \fn      Transition *kc::stop(Place *p, string p_name, string prefix)
 *
 * \brief   generates a subnet to stop
 *
 *          Generates a transition and places to stop the activity, i.e. a
 *          transition moving a token on the "stop" place to "stopped".
 *
 * \param   p  the place in control flow from which the token is move to
 *             "stop"
 *
 * \param   p_name  name of place p which is used to label the generated
 *                  transition (e.g. if p1 is called "running" generated
 *                  transition end with this name (e.g. "stop.running")
 *
 * \param   prefix  prefix of the pattern to label generated transition
 *
 * \return  a pointer to the stop transition
 *
 * \ingroup patterns
 */

#line  116 "petrinet-patterns-shortcuts.cc"
#line 258 "petrinet-patterns-shortcuts.k"
/*!
 * \fn      Transition *kc::stop_loop(Place *p, string p_name, string prefix)
 *
 * \brief   generates a subnet to stop
 *
 *          Generates a transition and places to stop the activity, i.e. a
 *          transition moving a token on the "stop" place to "stopped".
 *
 * \param   p  the place in control flow from which the token is move to
 *             "stop"
 *
 * \param   p_name  name of place p which is used to label the generated
 *                  transition (e.g. if p1 is called "running" generated
 *                  transition end with this name (e.g. "stop.running")
 *
 * \param   prefix  prefix of the pattern to label generated transition
 *
 * \return  a pointer to the stop transition
 *
 * \ingroup patterns
 */

#line  140 "petrinet-patterns-shortcuts.cc"
#line 301 "petrinet-patterns-shortcuts.k"
/*!
 * \fn      Transition *kc::stop_activity(Place *p, string p_name, string prefix)
 *
 * \brief   generates a subnet to stop
 *
 *          Generates a transition and places to stop the activity, i.e. a
 *          transition moving a token on the "stop" place to "stopped".
 *
 * \param   p  the place in control flow from which the token is move to
 *             "stop"
 *
 * \param   p_name  name of place p which is used to label the generated
 *                  transition (e.g. if p1 is called "running" generated
 *                  transition end with this name (e.g. "stop.running")
 *
 * \param   prefix  prefix of the pattern to label generated transition
 *
 * \return  a pointer to the stop transition
 *
 * \ingroup patterns
 */

#line  164 "petrinet-patterns-shortcuts.cc"
#line 345 "petrinet-patterns-shortcuts.k"
/*!
 * \fn      void kc::dpeLinks(Transition *t, int id)
 *
 * \brief   add a subnet for dead-path elimination
 *
 *          Creates arcs to set links on dead paths to false.
 *
 * \param   t   transition that invokes DPE
 * \param   id  the identifier of the AST node
 *
 * \ingroup patterns
 */

#line  179 "petrinet-patterns-shortcuts.cc"
#line 382 "petrinet-patterns-shortcuts.k"
/*!
 * \brief Adds a pattern for a cyclic net with a cap of loops
 *
 * \param   condition   the condition for entering/leaving the loop
 * \param   maxloops    the cap for the number of cycles
 * \param   prefix      the prefix of the places and transitions
 * \param   id          the id of the callin activity
 * \param   enterPlace  the where the loop is entered (needed to connect correctly)
 * \param   loopPlace   the place that is marked if the condition was accepted (needed to connect correctly)
 * \param   exitPlace   the place to be marked when the loop is left (needed to connect correctly)
 *
 * \ingroup patterns
 */

#line  195 "petrinet-patterns-shortcuts.cc"
#line 506 "petrinet-patterns-shortcuts.k"
/*!
 * \brief Adds a receiving pattern
 *
 * \param   name        the name of the communication transition
 * \param   prePlace    the place before the pattern (needed to connect correctly)
 * \param   postPlace   the place after the pattern (needed to connect correctly)
 * \param   channel     the place acting as channel place
 * \param   faultPlace  the place of the pattern, where faults are stored
 * \param   prefix      the prefix of the places and transitions
 * \param   id          the id of the callin activity
 * \param   negativeControlFlow  signals where the activity is located:
 *                                - 0: inside a scope or the process
 *                                - 1: inside a fault handler
 *                                - 2: inside a compensation handler
 *
 * \param   preventFurtherFaults  controls what happens to further faults
 *                                 - true: these faults are prevented (standard)
 *                                 - false: these faults are suppressed
 *
 * \ingroup patterns
 */

#line  219 "petrinet-patterns-shortcuts.cc"
#line 587 "petrinet-patterns-shortcuts.k"
/*!
 * \brief Adds a sending pattern
 *
 * \param   name        the name of the communication transition
 * \param   prePlace    the place before the pattern (needed to connect correctly)
 * \param   postPlace   the place after the pattern (needed to connect correctly)
 * \param   channel     the place acting as channel place
 * \param   faultPlace  the place of the pattern, where faults are stored
 * \param   prefix      the prefix of the places and transitions
 * \param   id          the id of the callin activity
 * \param   negativeControlFlow  signals where the activity is located:
 *                                - 0: inside a scope or the process
 *                                - 1: inside a fault handler
 *                                - 2: inside a compensation handler
 *
 * \param   preventFurtherFaults  controls what happens to further faults
 *                                 - true: these faults are prevented (standard)
 *                                 - false: these faults are suppressed
 *
 * \ingroup patterns
 */

#line  243 "petrinet-patterns-shortcuts.cc"
#line 662 "petrinet-patterns-shortcuts.k"
/*!
 * \fn void kc::P_transitionCondition(string prefix, tSource_list source_links)
 *
 * \brief   Pattern to model the transition condition of an activity
 *
 * \param   prefix        the prefix of the activity's pattern
 * \param   source_links  the list of outgoing links of the activity
 *
 * \ingroup patterns
 *
 * \todo    Implement nondeterministic transition condition and add respective
 *          parameter.
 */

#line  259 "petrinet-patterns-shortcuts.cc"
#line 761 "petrinet-patterns-shortcuts.k"
/*!
 * \fn void kc::P_joinCondition(string prefix, tTarget_list target_links, joinCondition jc)
 *
 * \brief   Pattern to model the join condition of an activity
 *
 * \param   prefix        the prefix of the activity's pattern
 * \param   target_links  the list of incoming links of the activity
 * \param   jc            the join condition
 *
 * \ingroup patterns
 */

#line  273 "petrinet-patterns-shortcuts.cc"
/* end included stuff */


namespace kc {

#ifndef KC_TRACE_PROVIDED
#define KC_TRACE_PROVIDED(COND,FILE,LINE,NODE) COND
#endif

Transition *throwFault(Place *p1, Place *p2, string p1name, string prefix, integer id, int negativeControlFlow, bool preventFurtherFaults)
#line 121 "petrinet-patterns-shortcuts.k"
{
    assert(p1 != NULL);
    assert(p2 != NULL);



    if (globals::parameters[P_COMMUNICATIONONLY])
    return NULL;




    if (globals::ASTEmap[globals::ASTEmap[id->value]->parentScopeId]->attributes["exitOnStandardFault"] == "yes")
    {
	Transition *t1 = PN.newTransition(prefix + "exitOnStandardFault." + p1name);
	PN.newArc(p1, t1);
	PN.newArc(t1, p2);
	PN.newArc(t1, PN.findPlace("1.internal.exit"));

	return t1;
    }



    switch (negativeControlFlow)
    {
	case(0): 
	{
	    unsigned int parentId = globals::ASTEmap[id->value]->parentScopeId;

	    Transition *t1 = PN.newTransition(prefix + "throwFault." + p1name);
	    PN.newArc(p1, t1);
	    PN.newArc(t1, p2);
	    PN.newArc(t1, PN.findPlace(toString(parentId) + ".internal.fault_in"));

	    return t1;
	}

	case(1): 
	case(4): 
	{


	    unsigned int parentId = globals::ASTEmap[globals::ASTEmap[id->value]->parentScopeId]->parentScopeId;

	    Transition *t1;
	    if (negativeControlFlow == 4)
	    t1 = PN.newTransition(prefix + "rethrow");
	    else
	    t1 = PN.newTransition(prefix + "rethrowFault." + p1name);

	    PN.newArc(p1, t1);
	    PN.newArc(t1, p2);

	    if (globals::ASTEmap[id->value]->parentScopeId == 1)
	    PN.newArc(t1, PN.findPlace(toString(parentId) + ".internal.exit"));
	    else
	    PN.newArc(t1, PN.findPlace(toString(parentId) + ".internal.fault_in"));

	    return t1;
	}

	case(2): 
	{
	    unsigned int parentId = globals::ASTEmap[id->value]->parentScopeId;


	    assert(parentId != 1);

	    Transition *t1 = PN.newTransition(prefix + "throwCHFault." + p1name);
	    PN.newArc(p1, t1);
	    PN.newArc(t1, p2);
	    PN.newArc(t1, PN.findPlace(toString(parentId) + ".internal.ch_fault_in"));

	    return t1;
	}

	case(3): 
	{
	    unsigned int parentId = globals::ASTEmap[id->value]->parentScopeId;

	    Transition *t1 = PN.newTransition(prefix + "signalFault." + p1name);
	    PN.newArc(p1, t1);
	    PN.newArc(t1, p2);
	    PN.newArc(t1, PN.findPlace(toString(parentId) + ".terminationHandler.inner_fault"));
	}
    }
    return NULL;

#line 209 "petrinet-patterns-shortcuts.k"
}
#line  376 "petrinet-patterns-shortcuts.cc"

Transition *stop(Place *p, string p_name, string prefix)
#line 239 "petrinet-patterns-shortcuts.k"
{
    assert(p != NULL);


    if (globals::parameters[P_COMMUNICATIONONLY])
    return NULL;

    Transition *stopTransition = PN.newTransition(prefix + "stopped." + p_name);
    PN.newArc(PN.findPlace(prefix + "stop"), stopTransition);
    PN.newArc(stopTransition, PN.findPlace(prefix + "stopped"));
    PN.newArc(p, stopTransition);

    return stopTransition;

#line 252 "petrinet-patterns-shortcuts.k"
}
#line  396 "petrinet-patterns-shortcuts.cc"

Transition *stop_loop(Place *p, string p_name, string prefix)
#line 281 "petrinet-patterns-shortcuts.k"
{
    assert(p != NULL);


    if (globals::parameters[P_COMMUNICATIONONLY])
    return NULL;

    Transition *stopTransition = PN.newTransition(prefix + "stopped_loop." + p_name);
    PN.newArc(PN.findPlace(prefix + "stop_loop"), stopTransition);
    PN.newArc(stopTransition, PN.findPlace(prefix + "stopped_loop"));
    PN.newArc(p, stopTransition);

    return stopTransition;

#line 294 "petrinet-patterns-shortcuts.k"
}
#line  416 "petrinet-patterns-shortcuts.cc"

Transition *stop_activity(Place *p, string p_name, string prefix)
#line 324 "petrinet-patterns-shortcuts.k"
{
    assert(p != NULL);


    if (globals::parameters[P_COMMUNICATIONONLY])
    return NULL;

    Transition *stopTransition = PN.newTransition(prefix + "stopped." + p_name);
    PN.newArc(PN.findPlace(prefix + "stop_activity"), stopTransition);
    PN.newArc(stopTransition, PN.findPlace(prefix + "stopped_activity"));
    PN.newArc(p, stopTransition);

    return stopTransition;

#line 337 "petrinet-patterns-shortcuts.k"
}
#line  436 "petrinet-patterns-shortcuts.cc"

void dpeLinks(Transition *t, int id)
#line 359 "petrinet-patterns-shortcuts.k"
{
    ENTER("[ASTT]");

    assert(t != NULL);
    assert(globals::ASTEmap[id] != NULL);

    for (set<unsigned int>::iterator linkID = globals::ASTEmap[id]->enclosedSourceLinks.begin();
	linkID != globals::ASTEmap[id]->enclosedSourceLinks.end();
	linkID++)
    {
	assert(globals::ASTEmap[*linkID] != NULL);
	string linkName = globals::ASTEmap[*linkID]->linkName;
	PN.newArc(t, PN.findPlace("!link." + linkName));
    }

    LEAVE("[ASTT]");

#line 375 "petrinet-patterns-shortcuts.k"
}
#line  459 "petrinet-patterns-shortcuts.cc"

void looppattern(string condition, int maxloops, string prefix, int id, Place *enterPlace, Place *loopPlace, Place *exitPlace)
#line 398 "petrinet-patterns-shortcuts.k"
{
    ENTER("[ASTT] looppattern");

    assert( enterPlace != NULL );
    assert( loopPlace != NULL );
    assert( exitPlace != NULL );

    std::string newPrefix = prefix + "loop.";

    int counter=0;

    if (condition == "false")
    {
	Place *Pbase = PN.newPlace(newPrefix + "count_" + toString(counter));
	Pbase->mark();

	Transition *t1 = PN.newTransition(newPrefix + "exit_" + toString(counter));
	PN.newArc(enterPlace, t1);
	PN.newArc(Pbase, t1);
	PN.newArc(t1, Pbase);
	PN.newArc(t1, exitPlace);

    } 
    else
    {
	if (condition == "true")
	{
	    Place *Pbase = PN.newPlace(newPrefix + "count_" + toString(counter));
	    Pbase->mark();
	    Place *Pcurrent = Pbase;
	    Transition *Tcurrent;
	    Transition *Tstop;

	    while(counter < maxloops)
	    {
		Tcurrent = PN.newTransition(newPrefix + "run_" + toString(counter+1));
		if (globals::parameters[P_LOOPCONTROL])
		PN.newArc(PN.findPlace("in.enterloop." + toString(id)), Tcurrent);
		PN.newArc(enterPlace, Tcurrent);
		PN.newArc(Pcurrent, Tcurrent);
		Pcurrent = PN.newPlace(newPrefix + "count_" + toString(counter+1));
		if (globals::parameters[P_COMMUNICATIONONLY] == false)
		{
		    Tstop = stop_loop(Pcurrent, "count_" + toString(counter+1), prefix);
		    PN.newArc(Tstop, Pbase);
		}
		PN.newArc(Tcurrent, Pcurrent);
		PN.newArc(Tcurrent, loopPlace);
		counter++;
	    }

	    Transition *Texit = PN.newTransition(newPrefix + "exit_" + toString(counter));
	    if (globals::parameters[P_LOOPCONTROL])
	    PN.newArc(PN.findPlace("in.leaveloop." + toString(id)), Texit);
	    PN.newArc(Pcurrent, Texit);
	    PN.newArc(enterPlace, Texit);
	    PN.newArc(Texit, Pbase);
	    PN.newArc(Texit, exitPlace);      
	}
	else
	{
	    Place *Pbase = PN.newPlace(newPrefix + "count_" + toString(counter));
	    Pbase->mark();
	    Transition *Texit = PN.newTransition(newPrefix + "exit_" + toString(counter));
	    if (globals::parameters[P_LOOPCONTROL])
	    PN.newArc(PN.findPlace("in.enterloop." + toString(id)), Texit);
	    PN.newArc(Pbase, Texit);
	    PN.newArc(enterPlace, Texit);
	    PN.newArc(Texit, Pbase);
	    PN.newArc(Texit, exitPlace);      

	    Place *Pcurrent =Pbase;
	    Transition *Tcurrent;
	    Transition *Tstop;

	    while(counter < maxloops)
	    {
		Tcurrent = PN.newTransition(newPrefix + "run_" + toString(counter+1));
		if (globals::parameters[P_LOOPCONTROL])
		PN.newArc(PN.findPlace("in.enterloop." + toString(id)), Tcurrent);
		PN.newArc(enterPlace, Tcurrent);
		PN.newArc(Pcurrent, Tcurrent);
		PN.newArc(Tcurrent, loopPlace);
		Pcurrent = PN.newPlace(newPrefix + "count_" + toString(counter+1));
		PN.newArc(Tcurrent, Pcurrent);
		if (globals::parameters[P_COMMUNICATIONONLY] == false)
		{
		    Tstop = stop_loop(Pcurrent, "count_" + toString(counter+1), prefix);
		    PN.newArc(Tstop, Pbase);
		}
		Tcurrent = PN.newTransition(newPrefix + "exit_" + toString(counter+1));
		if (globals::parameters[P_LOOPCONTROL])
		PN.newArc(PN.findPlace("in.leaveloop." + toString(id)), Tcurrent);
		PN.newArc(Pcurrent, Tcurrent);
		PN.newArc(enterPlace, Tcurrent);
		PN.newArc(Tcurrent, Pbase);
		PN.newArc(Tcurrent, exitPlace);
		counter++;
	    }
	}
    }
    LEAVE("[ASTT] looppattern");

#line 500 "petrinet-patterns-shortcuts.k"
}
#line  568 "petrinet-patterns-shortcuts.cc"

void receivingTransition(string name, Place *prePlace, Place *postPlace, Place *channel, Place *faultPlace, string prefix, integer id, int negativeControlFlow, bool preventFurtherFaults)
#line 531 "petrinet-patterns-shortcuts.k"
{
    ENTER("[ASTT] receivingTransition");

    assert( prePlace != NULL );
    assert( postPlace != NULL );
    assert( channel != NULL );
    assert( faultPlace != NULL );

    std::string newPrefix = prefix + name + ".";

    Place *p1 = PN.newPlace(newPrefix + "buffer");
    Place *p2 = PN.newPlace(newPrefix + "received");

    Transition *t1 = PN.newTransition(newPrefix + "receive");
    PN.newArc(prePlace, t1);
    PN.newArc(channel, t1);

    PN.newArc(t1, p1);

    Transition *t2 = PN.newTransition(newPrefix + "write_variable");
    PN.newArc(p1, t2);
    PN.newArc(t2, p2);

    if (globals::parameters[P_VARIABLES])
    {
	string varName = "variable." + globals::ASTEmap[id->value]->variableName;
	if ( name == "invoke_in" )
	{
	    varName = "variable." + globals::ASTEmap[id->value]->outputVariableName;
	}
	PN.newArc(PN.findPlace(varName), t2);
	PN.newArc(t2, PN.findPlace(varName));
    }

    Transition *t3 = PN.newTransition(newPrefix + "finish");
    PN.newArc(p2, t3);
    PN.newArc(t3, postPlace);

    Transition *t4 = stop(p1, name + ".buffer", prefix);
    Transition *t5 = stop(p2, name + ".received", prefix);

    if (globals::parameters[P_STANDARDFAULTS])
    {
	Transition *t6 = throwFault(p1, faultPlace, "buffer", newPrefix, id, globals::ASTEmap[id->value]->controlFlow, preventFurtherFaults);
	Transition *t7 = throwFault(p2, faultPlace, "received", newPrefix, id, globals::ASTEmap[id->value]->controlFlow, preventFurtherFaults);
    }


    LEAVE("[ASTT] receivingTransition");

#line 580 "petrinet-patterns-shortcuts.k"
}
#line  624 "petrinet-patterns-shortcuts.cc"

void sendingTransition(string name, Place *prePlace, Place *postPlace, Place *channel, Place *faultPlace, string prefix, integer id, int negativeControlFlow, bool preventFurtherFaults)
#line 612 "petrinet-patterns-shortcuts.k"
{
    ENTER("[ASTT] sendingTransition");

    std::string newPrefix = prefix + name + ".";

    Place *p1 = PN.newPlace(newPrefix + "buffer");
    Place *p2 = PN.newPlace(newPrefix + "sent");

    Transition *t1 = PN.newTransition(newPrefix + "read_variable");
    PN.newArc(prePlace, t1);
    PN.newArc(t1, p1);

    if (globals::parameters[P_VARIABLES])
    {
	string varName = "variable." + globals::ASTEmap[id->value]->variableName;
	if ( name == "invoke" || name == "invoke_out" )
	{
	    varName = "variable." + globals::ASTEmap[id->value]->inputVariableName;
	}
	PN.newArc(PN.findPlace(varName), t1, READ);
    }

    Transition *t2 = PN.newTransition(newPrefix + "send");
    PN.newArc(p1, t2);
    PN.newArc(t2, channel);

    PN.newArc(t2, p2);

    Transition *t3 = PN.newTransition(newPrefix + "finish");
    PN.newArc(p2, t3);
    PN.newArc(t3, postPlace);

    Transition *t4 = stop(p1, name + ".buffer", prefix);
    Transition *t5 = stop(p2, name + ".sent", prefix);

    if (globals::parameters[P_STANDARDFAULTS])
    {
	Transition *t6 = throwFault(p1, faultPlace, "buffer", newPrefix, id, globals::ASTEmap[id->value]->controlFlow, preventFurtherFaults);
	Transition *t7 = throwFault(p2, faultPlace, "sent", newPrefix, id, globals::ASTEmap[id->value]->controlFlow, preventFurtherFaults);
    }


    LEAVE("[ASTT] sendingTransition");

#line 655 "petrinet-patterns-shortcuts.k"
}
#line  674 "petrinet-patterns-shortcuts.cc"

void P_transitionCondition(string prefix, tSource_list source_links)
#line 677 "petrinet-patterns-shortcuts.k"
{



    Place *p_done = PN.findPlace(prefix + "done");
    Place *p_final = PN.findPlace(prefix + "final");

    assert(p_done != NULL);
    assert(p_final != NULL);






    if (globals::parameters[P_XOR] && (source_links->length() > 1))
    {

	{
#line 695 "petrinet-patterns-shortcuts.k"
	    tSource_list kc_fe_selvar_1 =  source_links;

#line  700 "petrinet-patterns-shortcuts.cc"
	    while(
		    kc_fe_selvar_1->prod_sel() == sel_ConstSource_list
		) {
		tSource kc_selvar_0_1 = kc_fe_selvar_1->tSource_1;
		{
#line 695 "petrinet-patterns-shortcuts.k"
		    {
			{
#line 696 "petrinet-patterns-shortcuts.k"
#line 695 "petrinet-patterns-shortcuts.k"
			    const tSource source = kc_selvar_0_1;
#line 696 "petrinet-patterns-shortcuts.k"

			    unsigned int linkCounter = 1;


			    Transition *t1 = PN.newTransition(prefix + "setLinks" + toString(linkCounter++));
			    PN.newArc(p_done, t1);
			    PN.newArc(t1, p_final);


			    Place *link_true = PN.findPlace("link." + globals::ASTEmap[source->id]->linkName);
			    PN.newArc(t1, link_true);


			    {
#line 709 "petrinet-patterns-shortcuts.k"
				tSource_list kc_fe_selvar_1 =  source_links;

#line  730 "petrinet-patterns-shortcuts.cc"
				while(
					kc_fe_selvar_1->prod_sel() == sel_ConstSource_list
				    ) {
				    tSource kc_selvar_1_1 = kc_fe_selvar_1->tSource_1;
				    {
#line 709 "petrinet-patterns-shortcuts.k"
					{
					    {
#line 710 "petrinet-patterns-shortcuts.k"
#line 709 "petrinet-patterns-shortcuts.k"
						const tSource source2 = kc_selvar_1_1;
#line 710 "petrinet-patterns-shortcuts.k"

						if (globals::ASTEmap[source2->id]->linkName != globals::ASTEmap[source->id]->linkName)
						{
						    Place *link_false = PN.findPlace("!link." + globals::ASTEmap[source2->id]->linkName);
						    PN.newArc(t1, link_false);
						}

#line  750 "petrinet-patterns-shortcuts.cc"
					    }
					}

#line  754 "petrinet-patterns-shortcuts.cc"
				    }
				    kc_fe_selvar_1 = kc_fe_selvar_1->tSource_list_1;
#line 709 "petrinet-patterns-shortcuts.k"

#line  759 "petrinet-patterns-shortcuts.cc"
				}
			    }
#line 717 "petrinet-patterns-shortcuts.k"


			    string givenTransitionCondition = globals::ASTEmap[source->id]->attributes["transitionCondition"];
			    if ((givenTransitionCondition != "") && (givenTransitionCondition != "true"))
			    {
				cerr << "Ignoring user-defined transition condition \"" << givenTransitionCondition << "\"" << endl;
				cerr << "Using XOR instead." << endl << endl;
			    }

#line  772 "petrinet-patterns-shortcuts.cc"
			}
		    }

#line  776 "petrinet-patterns-shortcuts.cc"
		}
		kc_fe_selvar_1 = kc_fe_selvar_1->tSource_list_1;
#line 695 "petrinet-patterns-shortcuts.k"

#line  781 "petrinet-patterns-shortcuts.cc"
	    }
	}
#line 726 "petrinet-patterns-shortcuts.k"
    }





    else
    {

	Transition *t1 = PN.newTransition(prefix + "setLinks");
	PN.newArc(p_done, t1);
	PN.newArc(t1, p_final);


	{
#line 740 "petrinet-patterns-shortcuts.k"
	    tSource_list kc_fe_selvar_1 =  source_links;

#line  803 "petrinet-patterns-shortcuts.cc"
	    while(
		    kc_fe_selvar_1->prod_sel() == sel_ConstSource_list
		) {
		tSource kc_selvar_0_1 = kc_fe_selvar_1->tSource_1;
		{
#line 740 "petrinet-patterns-shortcuts.k"
		    {
			{
#line 741 "petrinet-patterns-shortcuts.k"
#line 740 "petrinet-patterns-shortcuts.k"
			    const tSource source = kc_selvar_0_1;
#line 741 "petrinet-patterns-shortcuts.k"

			    Place *link_true = PN.findPlace("link." + globals::ASTEmap[source->id]->linkName);
			    PN.newArc(t1, link_true);


			    string givenTransitionCondition = globals::ASTEmap[source->id]->attributes["transitionCondition"];
			    if (givenTransitionCondition != "" && givenTransitionCondition != "true")
			    {
				cerr << "Ignoring user-defined transition condition \"" << givenTransitionCondition << "\"" << endl;
				cerr << "Using standard transition \"true\" instead." << endl << endl;
			    }

#line  828 "petrinet-patterns-shortcuts.cc"
			}
		    }

#line  832 "petrinet-patterns-shortcuts.cc"
		}
		kc_fe_selvar_1 = kc_fe_selvar_1->tSource_list_1;
#line 740 "petrinet-patterns-shortcuts.k"

#line  837 "petrinet-patterns-shortcuts.cc"
	    }
	}
#line 753 "petrinet-patterns-shortcuts.k"
    }

#line 754 "petrinet-patterns-shortcuts.k"
}
#line  845 "petrinet-patterns-shortcuts.cc"

void P_joinCondition(string prefix, tTarget_list target_links, joinCondition jc)
#line 774 "petrinet-patterns-shortcuts.k"
{

    map<unsigned int, Transition*> joinTransitions;
    map<unsigned int, Place*> trueLinks;
    map<unsigned int, Place*> falseLinks;
    unsigned int transitionNumber = (unsigned int)pow(2.0, target_links->length());

    Place *p_jc_false = PN.findPlace(prefix + "jc_false");
    Place *p_jc_true = PN.findPlace(prefix + "jc_true");

    assert(p_jc_false != NULL);
    assert(p_jc_true != NULL);




    unsigned int linkCount = 0;
    {
#line 791 "petrinet-patterns-shortcuts.k"
	tTarget_list kc_fe_selvar_1 =  target_links;

#line  870 "petrinet-patterns-shortcuts.cc"
	while(
		kc_fe_selvar_1->prod_sel() == sel_ConstTarget_list
	    ) {
	    tTarget kc_selvar_0_1 = kc_fe_selvar_1->tTarget_1;
	    {
#line 791 "petrinet-patterns-shortcuts.k"
		{
		    {
#line 792 "petrinet-patterns-shortcuts.k"
#line 791 "petrinet-patterns-shortcuts.k"
			const tTarget target = kc_selvar_0_1;
#line 792 "petrinet-patterns-shortcuts.k"

			Place *linkT = PN.findPlace("link." + globals::ASTEmap[target->id]->linkName);
			Place *linkF = PN.findPlace("!link." + globals::ASTEmap[target->id]->linkName);

			trueLinks[linkCount] = linkT;
			falseLinks[linkCount] = linkF;

			linkCount++;

#line  892 "petrinet-patterns-shortcuts.cc"
		    }
		}

#line  896 "petrinet-patterns-shortcuts.cc"
	    }
	    kc_fe_selvar_1 = kc_fe_selvar_1->tTarget_list_1;
#line 791 "petrinet-patterns-shortcuts.k"

#line  901 "petrinet-patterns-shortcuts.cc"
	}
    }
#line 801 "petrinet-patterns-shortcuts.k"



    for (unsigned int i = 0; i < transitionNumber; i++)
    {
	joinTransitions[i] = PN.newTransition(prefix + "join" + toString(i));

	for (unsigned int j = 0; j < linkCount; j++)
	{
	    if (((i >> j) % 2) == 0) 
	    PN.newArc(trueLinks[j], joinTransitions[i]);
	    else
	    PN.newArc(falseLinks[j], joinTransitions[i]);  
	}
    }






    if (string(jc->op_name()) == "standardJoinCondition")
    {

	for (unsigned int i = 0; i < transitionNumber; i++)
	{


	    if (i == transitionNumber-1)
	    PN.newArc(joinTransitions[i], p_jc_false);
	    else
	    PN.newArc(joinTransitions[i], p_jc_true);
	}
    }






    else
    {

	for (unsigned int i = 0; i < transitionNumber; i++)
	{

	    if (globals::joinCondition_result[jc->parentId][i])
	    PN.newArc(joinTransitions[i], p_jc_true);
	    else
	    PN.newArc(joinTransitions[i], p_jc_false);
	}  
    }

#line 853 "petrinet-patterns-shortcuts.k"
}
#line  960 "petrinet-patterns-shortcuts.cc"


} // namespace kc
